<?php
class Musexpress_Admin {
    public static $musexpress_installed_plugins;

    public function __construct() {
        self::$musexpress_installed_plugins = array();
    }

    public function add_musegain_id_to_download_request($update) {
        $musegain_id = get_option('musegain_id');

        if (!empty($musegain_id)) {
            $update->download_url = add_query_arg('musegain_id', $musegain_id, $update->download_url);
        }

        return $update;
    }

	public function enqueue_styles() {
		wp_enqueue_style(MUSEXPRESS_PLUGIN_NAME, plugin_dir_url(__FILE__) . 'css/musexpress-admin.css', array(), MUSEXPRESS_PLUGIN_VERSION, 'all');
	}

    public function musexpress_admin_menu() {
        require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'admin/partials/musexpress-admin-display.php';
        require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'admin/partials/musexpress-admin-settings-display.php';
        require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'admin/partials/musexpress-page-converter-display.php';
        require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'admin/partials/musexpress-official-plugins-display.php';

        add_menu_page('MusexPress', 'MusexPress', 'manage_options', MUSEXPRESS_TOP_LEVEL_MENU_SLUG, 'musexpress_admin_display', plugin_dir_url(__FILE__) . '/img/menu_icon.png');

        $musegain_id = get_option('musegain_id');

        if (!empty($musegain_id)) {
            add_submenu_page(MUSEXPRESS_TOP_LEVEL_MENU_SLUG, 'Official Plugins', 'Official Plugins', 'manage_options', MUSEXPRESS_OFFICIAL_PLUGINS_PAGE_SLUG, 'musexpress_official_plugins_display');
        }

        add_submenu_page(MUSEXPRESS_TOP_LEVEL_MENU_SLUG, 'Pages Converter', 'Pages Converter', 'manage_options', MUSEXPRESS_PAGE_CONVERTER_PAGE_SLUG, 'musexpress_page_converter_display');
        add_submenu_page(MUSEXPRESS_TOP_LEVEL_MENU_SLUG, 'Settings', 'Settings', 'manage_options', MUSEXPRESS_SETTINGS_PAGE_SLUG, 'musexpress_admin_settings_display');
    }

    public function musexpress_admin_notices() {
        $musegain_id = get_option('musegain_id');

        if (empty($musegain_id)) {
            $this->musexpress_print_admin_notice('Please <a href="' . menu_page_url(MUSEXPRESS_SETTINGS_PAGE_SLUG, false) . '">verify</a> your <i>"MuseGain ID"</i> to enable MusexPress automatic updates!', 'info');
        }

        if (isset($_GET['musexpress_notice'])) {
            switch ($_GET['musexpress_notice']) {
                case 'verify-musegain-id':
                    $message = 'Your <i>"MuseGain ID"</i> has been successfully verified.';
                    $status = 'success';
                    $is_dismissible = true;

                    break;

                case 'delete-musegain-id':
                    $message = 'Your <i>"MuseGain ID"</i> has been successfully deleted.';
                    $status = 'success';
                    $is_dismissible = true;

                    break;

                case 'update-page-converter-settings':
                    $message = 'Settings saved.';
                    $status = 'success';
                    $is_dismissible = true;

                    break;

                case 'musexpress-page-converter':
                    $message = 'Pages converted successfully.';
                    $status = 'success';
                    $is_dismissible = true;

                    break;
            }

            $this->musexpress_print_admin_notice($message, $status, $is_dismissible);
        }
    }

    public function musexpress_enable_updates() {
        require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'includes/plugin-update-checker/plugin-update-checker.php';

        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        foreach ($this->get_musexpress_plugins() as $plugin) {
            $plugin_filename = $plugin['filename'];
            $plugin_slug = $plugin['slug'];

            if (array_key_exists($plugin_filename, get_plugins())) {
                self::$musexpress_installed_plugins[] = $plugin_filename;

                PucFactory::buildUpdateChecker(MUSEXPRESS_UPDATE_SERVER_URL . "/?action=get_metadata&slug=$plugin_slug", WP_PLUGIN_DIR . "/$plugin_filename", $plugin_slug);

                add_filter("puc_pre_inject_update-$plugin_slug", array($this, 'add_musegain_id_to_download_request'));
            }
        }
    }

    public function musexpress_plugin_row_meta($links, $file) {
        foreach ($this->get_musexpress_plugins() as $plugin) {
            if ($plugin['filename'] === $file) {
                $url_query_args = array(
                    'musexpress_action'  => 'musexpress_download_widgets',
                    'musexpress_plugin' => $plugin['slug']
                );

                $url_query_args['nonce'] = wp_create_nonce($url_query_args['musexpress_action'] . '_nonce');

                $links['musexpress_download_widgets'] = '<a href="' . esc_url_raw(add_query_arg($url_query_args), MUSEXPRESS_PLUGIN_DIRECTORY_PATH) . '">Download Widgets</a>';
            }
        }

        return $links;
    }

    public function musexpress_welcome_panel() {
        require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'admin/partials/musexpress-welcome-panel.php';
    }

    public static function get_musexpress_plugins() {
        $musexpress_json_file = MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'musexpress.json';

        $musexpress_plugins = array();

        if (file_exists($musexpress_json_file)) {
            foreach (json_decode(file_get_contents($musexpress_json_file)) as $musexpress_plugin) {
                $slug = $musexpress_plugin->slug;

                $musexpress_plugins[] = array(
                    'slug'              => $slug,
                    'filename'          => "$slug/$slug.php",
                    'name'              => $musexpress_plugin->name,
                    'description'       => $musexpress_plugin->description,
                    'version'           => $musexpress_plugin->version,
                    'documentation_uri' => $musexpress_plugin->documentation_uri,
                    'image_uri'         => $musexpress_plugin->image_uri,
                    'plugin_uri'        => $musexpress_plugin->plugin_uri
                );
            }
        }

        return $musexpress_plugins;
    }

    private function musexpress_print_admin_notice($message, $status, $is_dismissible = false) {
        echo '<div class="notice notice-' . $status . ($is_dismissible ? ' is-dismissible' : null) . '"><p>' . $message . '</p></div>';
    }

    public function process_data() {
        if (isset($_REQUEST['musexpress_action']) && wp_verify_nonce($_REQUEST['nonce'], $_REQUEST['musexpress_action'] . '_nonce')) {
            switch ($_REQUEST['musexpress_action']) {
                case 'verify_musegain_id':
                    $musegain_id = sanitize_email($_POST['musegain_id']);

                    if (!empty($musegain_id)) {
                        do_action('verify_musegain_id', $musegain_id);
                    }

                    break;

                case 'delete_musegain_id':
                    update_option('musegain_id', '');
                    wp_safe_redirect(esc_url_raw(add_query_arg('musexpress_notice', 'delete-musegain-id', menu_page_url(MUSEXPRESS_TOP_LEVEL_MENU_SLUG, false))));

                    break;

                case 'musexpress_download_widgets':
                    if(ini_get('zlib.output_compression')) {
            			@ini_set('zlib.output_compression', 'Off');
            		}

                    $musexpress_plugin = $_GET['musexpress_plugin'];

                    $filename = plugin_dir_path(WP_PLUGIN_DIR . "/$musexpress_plugin/$musexpress_plugin.php") . 'widgets.zip';

                    header('Content-Disposition: attachment; filename="MusexPress - ' . ucwords(implode(' ', explode('-', $musexpress_plugin))) . ' widgets.zip"');
                    header('Content-Length: ' . filesize($filename));
                    header('Content-Transfer-Encoding: binary');
            		header('Content-Type: application/zip');

            		readfile($filename);

                    break;

                case 'musexpress_page_converter':
                    require_once MUSEXPRESS_PLUGIN_DIRECTORY_PATH . 'admin/class-musexpress-page-converter.php';

                    $desktop_pages = isset($_POST['musexpress_page_converter_desktop_pages']) ? $_POST['musexpress_page_converter_desktop_pages'] : array();
                    $tablet_pages = isset($_POST['musexpress_page_converter_tablet_pages']) ? $_POST['musexpress_page_converter_tablet_pages'] : array();
                    $phone_pages = isset($_POST['musexpress_page_converter_phone_pages']) ? $_POST['musexpress_page_converter_phone_pages'] : array();

                    $page_converter = new Musexpress_Page_Converter($desktop_pages, $tablet_pages, $phone_pages, get_home_url(), get_option('page_converter_settings'));

                    $page_converter->init();

                    wp_safe_redirect(add_query_arg('musexpress_notice', 'musexpress-page-converter'));

                    break;

                case 'update_official_plugins_list':
                    do_action('get_musexpress_plugins');

                    break;

                case 'update_page_converter_settings':
                    $page_converter_settings = get_option('page_converter_settings');

                    foreach ($page_converter_settings as $settings_name => $settings) {
                        $page_converter_settings[$settings_name]['value'] = isset($_POST['page_converter_settings'][$settings_name]);

                    }

                    update_option('page_converter_settings', $page_converter_settings);
                    wp_safe_redirect(add_query_arg('musexpress_notice', 'update-page-converter-settings'));

                    break;
            }
        }
    }
}
