<?php
class Musexpress_Page_Converter {
    private $desktop_files;
    private $tablet_files;
    private $phone_files;
    private $base_url;

    private $add_wp_function;
    private $add_parser;
    private $convert_internal_url;
    private $absolute_css_muse_url;
    private $absolute_script_muse_url;
    private $absolute_images_muse_url;
    private $wp_head_injection;
    private $wp_footer_injection;
    private $sitemap;
    private $muse_manifest;
    private $add_files_in_phone_version;
    private $add_files_in_tablet_version;
    private $convert_redirect_version_url;
    private $create_robots_file;
    private $create_index_php_priority;
    private $muse_jquery;

    private $root_path;
    private $log;

    public function __construct($desktop_files, $tablet_files, $phone_files, $base_url, $page_converter_settings) {
        require_once ABSPATH . 'vendor/simple_html_dom.php';

        $this->desktop_files = $desktop_files;
        $this->tablet_files = $tablet_files;
        $this->phone_files = $phone_files;
        $this->base_url = $base_url;

        $this->add_wp_function = $page_converter_settings['add_wp_function']['value'];
        $this->add_parser = $page_converter_settings['add_parser']['value'];
        $this->convert_internal_url = $page_converter_settings['convert_internal_url']['value'];
        $this->absolute_css_muse_url = $page_converter_settings['absolute_css_muse_url']['value'];
        $this->absolute_script_muse_url = $page_converter_settings['absolute_script_muse_url']['value'];
        $this->absolute_images_muse_url = $page_converter_settings['absolute_images_muse_url']['value'];
        $this->wp_head_injection = $page_converter_settings['wp_head_injection']['value'];
        $this->wp_footer_injection = $page_converter_settings['wp_footer_injection']['value'];
        $this->sitemap = $page_converter_settings['sitemap']['value'];
        $this->muse_manifest = $page_converter_settings['muse_manifest']['value'];
        $this->add_files_in_phone_version = $page_converter_settings['add_files_in_phone_version']['value'];
        $this->add_files_in_tablet_version = $page_converter_settings['add_files_in_tablet_version']['value'];
        $this->convert_redirect_version_url = $page_converter_settings['convert_redirect_version_url']['value'];
        $this->create_robots_file = $page_converter_settings['create_robots_file']['value'];
        $this->create_index_php_priority = $page_converter_settings['create_index_php_priority']['value'];
        $this->muse_jquery = $page_converter_settings['muse_jquery']['value'];

        $this->root_path = ABSPATH;
        $this->log = array();
    }

    private function delete_all_converted_pages($root) {
        if (file_exists($root . 'conv_pages.log')) {
            $data = file_get_contents($root . 'conv_pages.log');
            $convert = explode("\n", $data);

            unlink($root . 'conv_pages.log');

        }

        $log = ABSPATH . 'conv_pages.json';

        if (file_exists($log)) {
            $converted_pages = json_decode(file_get_contents($log), true);

            foreach ($converted_pages as $converted_page => $mtime) {
                $php_file = ABSPATH . str_replace('.html', '.php', $converted_page);

                if (!file_exists(ABSPATH . $converted_page)) {
                    unset($converted_pages[$converted_page]);

                    if (file_exists($php_file)) {
                        unlink(trim($php_file));
                    }
                }
            }

            file_put_contents($log, json_encode($converted_pages, JSON_PRETTY_PRINT));
        }
    }

    private function index_php_priority($create_index_php_priority) {
        if($create_index_php_priority) {
            $htaccess_filename = $this->root_path . '.htaccess';

            if(file_exists($htaccess_filename)) {
                if(!strpos(file_get_contents($htaccess_filename), 'DirectoryIndex index.php index.html index.cgi') !== false) {
                    file_put_contents($htaccess_filename, PHP_EOL . 'DirectoryIndex index.php index.html index.cgi' . PHP_EOL, FILE_APPEND);
                }
            }
        }
    }

    private function prevent_musexpress_download() {
$htaccess = <<<EOT
<Files "musexpress.zip">
Order Allow,Deny
Deny from all
</Files>
EOT;

        $assets_path = ABSPATH . '/assets/';

        if (file_exists($assets_path)) {
            file_put_contents($assets_path . '.htaccess', $htaccess);
        }
    }

    public function init() {
        $this->index_php_priority($this->create_index_php_priority);

        $html_pages_list = $this->desktop_files;

        $this->delete_all_converted_pages($this->root_path);

        foreach ($html_pages_list as $original_page_name) {
            $converted = $this->page_conversion($this->base_url, $this->root_path . $original_page_name, $html_pages_list, $this->wp_footer_injection, $this->wp_head_injection, $this->absolute_images_muse_url, $this->absolute_script_muse_url, $this->absolute_css_muse_url, $this->convert_internal_url, $this->add_parser, $this->add_wp_function, $this->convert_redirect_version_url, $this->muse_jquery);

            if ($converted === true) {
                $this->write_all_converted_pages($this->root_path, $original_page_name, $this->log);
                $this->sitemap_conversion($this->sitemap, $this->root_path . 'sitemap.xml', $original_page_name);
                $this->manifest_conversion($this->muse_manifest, $this->root_path . 'muse_manifest.xml', $original_page_name);
            }
        }

        $this->delete_all_converted_pages($this->root_path . 'tablet/');

        $html_pages_list = $this->tablet_files;

        foreach ($html_pages_list as $original_page_name) {
            $converted = $this->page_conversion($this->base_url, $this->root_path .'tablet/'. $original_page_name, $html_pages_list, $this->wp_footer_injection, $this->wp_head_injection, $this->absolute_images_muse_url, $this->absolute_script_muse_url, $this->absolute_css_muse_url, $this->convert_internal_url, $this->add_parser, $this->add_wp_function, $this->convert_redirect_version_url, $this->muse_jquery);

            if ($converted === true) {
                $this->write_all_converted_pages($this->root_path . 'tablet/', $original_page_name, $this->log);
            }
        }

        $this->delete_all_converted_pages($this->root_path . 'phone/');

        $html_pages_list = $this->phone_files;

        foreach ($html_pages_list as $original_page_name) {
            $converted = $this->page_conversion($this->base_url, $this->root_path .'phone/'. $original_page_name, $html_pages_list, $this->wp_footer_injection, $this->wp_head_injection, $this->absolute_images_muse_url, $this->absolute_script_muse_url, $this->absolute_css_muse_url, $this->convert_internal_url, $this->add_parser, $this->add_wp_function, $this->convert_redirect_version_url, $this->muse_jquery);

            if ($converted === true) {
                $this->write_all_converted_pages($this->root_path . 'phone/', $original_page_name, $this->log);
            }
        }

        if($this->create_robots_file){
            $this->write_robots_txt($this->root_path);
        }

        if($this->add_files_in_phone_version) {
            if (file_exists($this->root_path . 'phone/')) {
$wp_load_mod = <<<EOT
<?php
require_once("../wp-load.php");
?>
EOT;

$simple_html_dom_mod = <<<EOT
<?php
include_once("../vendor/simple_html_dom.php");
?>
EOT;

            if (file_exists($this->root_path . 'phone/vendor/')) {
                file_put_contents($this->root_path . 'phone/vendor/simple_html_dom.php', $simple_html_dom_mod);
            } else {
                mkdir($this->root_path . "phone/vendor");
                file_put_contents($this->root_path . 'phone/vendor/simple_html_dom.php', $simple_html_dom_mod);
            }

            file_put_contents($this->root_path . 'phone/wp-load.php', $wp_load_mod);

            }
        }

        if ($this->add_files_in_tablet_version) {
            if (file_exists($this->root_path . 'tablet/')) {

$wp_load_mod = <<<EOT
<?php
   require_once("../wp-load.php");
?>
EOT;

$simple_html_dom_mod = <<<EOT
<?php
include_once("../vendor/simple_html_dom.php");
?>
EOT;

            if (file_exists($this->root_path . 'tablet/vendor/')) {
                file_put_contents($this->root_path .'tablet/vendor/simple_html_dom.php', $simple_html_dom_mod);
            }else{
                mkdir($this->root_path . "tablet/vendor");
                file_put_contents($this->root_path . 'tablet/vendor/simple_html_dom.php', $simple_html_dom_mod);
            }

            file_put_contents($this->root_path . 'tablet/wp-load.php', $wp_load_mod);

            }
        }

        $this->prevent_musexpress_download();
    }

    private function page_conversion($base_url, $original_page_name, $html_pages_list, $wp_footer_injection, $wp_head_injection, $absolute_images_muse_url, $absolute_script_muse_url, $absolute_css_muse_url, $convert_internal_url, $add_parser, $add_wp_function, $convert_redirect_version_url, $muse_jquery) {
        $output_page_name = str_replace('.html', '.php', $original_page_name);

        $page = $original_page_name;

        $page_name = str_replace('.html', '', $page);
        $page_contents = file_get_contents($page);

$pattern = <<<'LOD'
~

#definitions
(?(DEFINE)
    (?<sq> '(?>[^'\\]+|\\.)*+(?>'|\z) ) # content inside simple quotes
    (?<dq> "(?>[^"\\]+|\\.)*+(?>"|\z) ) # content inside double quotes
    (?<vn> [a-zA-Z_]\w*+ ) # variable name
    (?<crlf> \r?\n ) # CRLF
    (?<hndoc> <<< (["']?) (\g<vn>) \g{-2} \g<crlf> # content inside here/nowdoc
              (?> [^\r\n]+ | \R+ (?!\g{-1}; $) )*+
              (?: \g<crlf> \g{-1}; \g<crlf> | \z )
    )
    (?<cmt> /\*                      # multiline comments
             (?> [^*]+ | \* (?!/) )*+
             \*/
    )
)

#pattern
<\?php \s+
(?> [^"'?/<]+ | \?+(?!>) | \g<sq> | \g<dq> | \g<hndoc> | \g<cmt> | [</]+ )*+
(?: \?> | \z )

~xsm
LOD;

        $matches = array();

        $page = preg_replace_callback($pattern, function ($match) {
            global $matches;

            $matches[] = $match;

            return '[MUSEGAIN-CODE#'.max(array_keys($matches)).']';
        }, $page_contents);

        $page_parse = str_get_html($page, true, true, DEFAULT_TARGET_CHARSET, false, DEFAULT_BR_TEXT, DEFAULT_SPAN_TEXT);

        if ($page_parse === false) {
            return false;
        }

        if ($absolute_css_muse_url) {
            foreach ($page_parse->find('link') as $element) {
                if (substr($element->href, 0, 4) === 'css/') {
                    $element->href =  $base_url . '/'. $element->href;
                }
            }
        }

        if ($absolute_images_muse_url) {
            foreach ($page_parse->find('link') as $element) {
                if (substr($element->href, 0, 7) === 'images/') {
                    $element->href =  $base_url . '/'. $element->href;
                }

                if (substr($element->href, 0, 12) === 'wp-includes/') {
                    $element->href =  $base_url . '/'. $element->href;
                }
            }
        }

        if ($absolute_script_muse_url) {
            foreach ($page_parse->find('script') as $element) {
                if (substr($element->src, 0, 8) === 'scripts/') {
                    $element->src =  $base_url . '/'. $element->src;
                }

                if (substr($element->src, 0, 12) === 'wp-includes/') {
                    $element->src =  $base_url . '/'. $element->src;
                }
            }
        }

        if ($convert_internal_url) {

            foreach ($page_parse->find('a') as $element) {

                if (strpos($element->href, $base_url) !== false) {
                    $element->href = str_replace('.html', '.php', $element->href);

                }

                if (in_array(strstr($element->href, '#', true), $html_pages_list)) {
                    $element->href = str_replace('.html', '.php', $element->href);

                }

                if (in_array($element->href, $html_pages_list)) {
                    $element->href = str_replace('.html', '.php', $element->href);

                }
            }
        }

        if ($convert_redirect_version_url) {
            $page_name = basename($page_name);

            foreach ($page_parse->find('link[rel="alternate"]') as $element) {
                $element->href = str_replace('phone/' . $page_name . '.html', 'phone/' . $page_name . '.php', $element->href);
                $element->href = str_replace('tablet/' . $page_name . '.html', 'tablet/' . $page_name . '.php', $element->href);
            }

            foreach ($page_parse->find('head', 0)->find('script') as $element) {
                $element->innertext = str_replace('phone/' . $page_name . '.html', 'phone/' . $page_name . '.php', $element->innertext);
                $element->innertext = str_replace('tablet/' . $page_name . '.html', 'tablet/' . $page_name . '.php', $element->innertext);
            }
        }

        if ($wp_head_injection) {
$inject_head = <<<EOT
<?php
wp_head();
?>
EOT;

            $page_parse->find('head', 0)->innertext = $page_parse->find('head', 0)->innertext.$inject_head;
        }

        if ($wp_footer_injection) {
$inject_body = <<<EOT
<?php
wp_footer();
?>
EOT;

            $page_parse->find('body', 0)->innertext = $page_parse->find('body', 0)->innertext.$inject_body;
        }

        $html_result = $page_parse;

        $pattern = '/\[MUSEGAIN-CODE\#([\d]+)\]/';

        $php_page = preg_replace_callback($pattern, function ($match) use($matches) {
            global $matches;

            return $matches[$match[1]][0];
        }, $html_result);

        $add_wp_function_code = '';

        if ($add_wp_function) {
$add_wp_function_code = <<<EOT
require_once("wp-load.php");
wp();
do_action('template_redirect');
apply_filters('template_include', get_index_template());
EOT;
        }

        $add_parser_code = '';

        if ($add_parser) {
$add_parser_code = <<<EOT
include_once('vendor/simple_html_dom.php');
EOT;
        }

$php_page = <<<EOT
<?php
$add_wp_function_code
$add_parser_code
?>
$php_page
EOT;

if ($muse_jquery) {
    $php_page = str_replace("if (document.location.protocol != 'https:') document.write('\\x3Cscript src=\"http://musecdn2.businesscatalyst.com/scripts/4.0/jquery-1.8.3.min.js\" type=\"text/javascript\">\\x3C/script>');", "", $php_page);
    $php_page = str_replace("window.jQuery || document.write('\\x3Cscript src=\"scripts/jquery-1.8.3.min.js\" type=\"text/javascript\">\\x3C/script>');", "", $php_page);
}

        file_put_contents($output_page_name, $php_page);

        return true;
    }

    private function write_all_converted_pages($root, $page, &$log) {
        $page_php = str_replace('html', 'php', $page);

        file_put_contents($root . 'conv_pages.log', $page_php . PHP_EOL, FILE_APPEND);

        /*if($create_robots_file && $root == $this->root_path) {
            if (!file_exists($root . 'robots.txt')) {
                file_put_contents($root . 'robots.txt', 'User-Agent: *' . PHP_EOL);
            }

            file_put_contents($root . 'robots.txt', 'Disallow: /' . $page . PHP_EOL, FILE_APPEND);
        }*/

        $log_json = ABSPATH . 'conv_pages.json';

        if (file_exists($log_json)) {
            $log = json_decode(file_get_contents($log_json), true);
        }

        $log[str_replace(ABSPATH, '', $root) . basename($page)] = array(
            'mtime' => filemtime($root . $page)
        );

        file_put_contents(ABSPATH . 'conv_pages.json', json_encode($log, JSON_PRETTY_PRINT));
    }

    private function write_robots_txt($root){

            $robot_filename = $root . 'robots.txt';

            if (file_exists($robot_filename)) {
                unlink($robot_filename);
            }

            file_put_contents($root . 'robots.txt', 'User-Agent: *' . PHP_EOL);

            $log_json = $root . 'conv_pages.json';

            if (file_exists($log_json)) {
                $converted_pages = json_decode(file_get_contents($log_json), true);

                foreach ($converted_pages as $converted_page => $mtime) {
                    file_put_contents($root . 'robots.txt', 'Disallow: /' . $converted_page . PHP_EOL, FILE_APPEND);
                }
            }
    }

    private function sitemap_conversion($sitemap, $sitemapURL, $original_page_name) {
        if ($sitemap) {
            $page_sitemap = file_get_contents($sitemapURL);

            $doc = new DOMDocument();
            $doc->preserveWhiteSpace = false;
            $doc->formatOutput = true;
            $doc->loadXML($page_sitemap);

            $xml_url = $doc->getElementsByTagName('loc');
            foreach ($xml_url as $url) {
                if (strpos($url->nodeValue, $original_page_name) !== false) {
                    $url->nodeValue = str_replace('html', 'php', $url->nodeValue);
                }
            }

            $xml_string = $doc->saveXML();

            file_put_contents($sitemapURL, $xml_string);
        }
    }

    private function manifest_conversion($muse_manifest, $manifestURL, $original_page_name) {
        if($muse_manifest){
            $page_muse_manifest = file_get_contents($manifestURL);

            $doc = new DOMDocument();
            $doc->preserveWhiteSpace = false;
            $doc->formatOutput = true;
            $doc->loadXML($page_muse_manifest);

            $xml_url = $doc->getElementsByTagName('file');
            foreach ($xml_url as $url) {
                if(strpos($url->getAttribute('name'), $original_page_name) !== false){
                 $url->setAttribute("name", str_replace('html', 'php', $url->getAttribute('name')));
                }
            }

            $xml_string = $doc->saveXML();

            file_put_contents($manifestURL, $xml_string);
        }
        //END: conversione file muse_manifest.xml generato da MUSE
    }
}
